package edu.prlab.tyler.iotgateway.cloud;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import edu.prlab.tyler.iotgateway.cloud.pojo.auth.SensitiveUser;
import edu.prlab.tyler.iotgateway.cloud.pojo.device.Device;
import edu.prlab.tyler.iotgateway.cloud.pojo.device.Icon;
import edu.prlab.tyler.iotgateway.cloud.pojo.device.Model;
import edu.prlab.tyler.iotgateway.cloud.pojo.privacy.PrivacyChoice;
import edu.prlab.tyler.iotgateway.cloud.pojo.privacy.PrivacyContent;
import edu.prlab.tyler.iotgateway.cloud.pojo.privacy.PrivacyPolicyReport;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.web.context.WebApplicationContext;

import java.util.ArrayList;
import java.util.Arrays;

import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = CloudApplication.class)
public class CloudHttpApiTest {
    @Autowired
    WebApplicationContext context;

    private MockMvc mvc;
    private ObjectMapper mapper;

    @Before
    public void setup() {
        this.mvc = MockMvcBuilders
                .webAppContextSetup(context)
                .build();
        this.mapper = new ObjectMapper();
        mapper.configure(DeserializationFeature.ACCEPT_SINGLE_VALUE_AS_ARRAY, true);
    }

    @Test
    public void addAndGetDevice() throws Exception {

        Device device = Device.builder()
                .udn("udn")
                .name("name")
                .type(Device.Type.Sensor)
                .UPC("upc")
                .model(Model.builder()
                        .name("name")
                        .url("url")
                        .description("description")
                        .build())
                .location("location")
                .icons(Arrays.asList(Icon.builder()
                        .build()))
                .status(Device.Status.Disconnected)
                .build();

        MvcResult result = mvc.perform(MockMvcRequestBuilders
                .post("/device")
                .contentType(MediaType.APPLICATION_JSON_UTF8)
                .accept(MediaType.APPLICATION_JSON_UTF8)
                .content(mapper.writeValueAsString(device)))
                .andDo(print())
                .andExpect(status().isOk())
                .andReturn();
        device = mapper.readValue(result.getResponse().getContentAsString(), Device.class);
        Assert.assertNotNull(device);

        result = mvc.perform(MockMvcRequestBuilders
                .get("/device/" + device.getId())
                .accept(MediaType.APPLICATION_JSON_UTF8))
                .andDo(print())
                .andExpect(status().isOk())
                .andReturn();
        device = mapper.readValue(result.getResponse().getContentAsString(), Device.class);
        Assert.assertNotNull(device);

        result = mvc.perform(MockMvcRequestBuilders
                .get("/device")
                .accept(MediaType.APPLICATION_JSON_UTF8))
                .andDo(print())
                .andExpect(status().isOk())
                .andReturn();
        ArrayList<Device> devices = mapper.readValue(result.getResponse().getContentAsString(),
                new TypeReference<ArrayList<Device>>() {});
        Assert.assertNotNull(devices);

        result = mvc.perform(MockMvcRequestBuilders
                .get("/device/privacy/" + device.getUdn())
                .accept(MediaType.APPLICATION_JSON_UTF8))
                .andDo(print())
                .andExpect(status().isOk())
                .andReturn();
        PrivacyPolicyReport report = mapper.readValue(result.getResponse().getContentAsString(), PrivacyPolicyReport.class);
        Assert.assertNotNull(report);

        PrivacyChoice privacyChoice = PrivacyChoice.builder()
                .privacyContent(PrivacyContent.builder()
                        .user(SensitiveUser.builder()
                                .username("test")
                                .password("test")
                                .build())
                        .device(device)
                        .policy(report.getPolicies().get(0))
                        .build())
                .isAccepted(true)
                .build();

        result = mvc.perform(MockMvcRequestBuilders
                .post("/choice")
                .contentType(MediaType.APPLICATION_JSON_UTF8)
                .accept(MediaType.APPLICATION_JSON_UTF8)
                .content(mapper.writeValueAsString(privacyChoice)))
                .andDo(print())
                .andExpect(status().isOk())
                .andReturn();

        Assert.assertNotNull(result);
    }
}
